#!/usr/bin/env node
import { cli } from 'cleye';
import { promisify } from 'node:util';
import { exec } from 'node:child_process';
import { select, text, intro, group, cancel, confirm, outro } from '@clack/prompts';
import { dim, red, green, blue, lightYellow } from 'kolorist';
import dedent from 'dedent';

var name = "commitizen-cli";
var version = "1.1.0";
var author = {
	name: "iamando",
	email: "iamando@tuta.io"
};

const PACKAGE_NAME = name;
const VERSION = version;
const AUTHOR = author;
const CANCELED_OP_MSG = "Operation Canceled";

const type = () => select({
  message: "Choose commit type",
  options: [
    { label: "Feature", value: "feature" },
    { label: "Bugfix", value: "bugfix" },
    { label: "Hotfix", value: "hotfix" },
    { label: "Chore", value: "chore" },
    { label: "Epic", value: "epic" },
    { label: "Design", value: "design" },
    { label: "Experiment", value: "experiment" }
  ]
});
const message = () => text({
  message: "Insert commit message",
  validate: (value) => {
    if (value.length === 0)
      return "Commit message required";
  }
});

const log = ({
  type,
  msg,
  isConsole = true,
  newLine = true
}) => {
  switch (type) {
    case "info":
      return isConsole ? console.info(`${newLine ? "\n" : ""} ${blue("\u2754")} ${msg}`) : `${newLine ? "\n" : ""} ${blue("\u2754")} ${msg}`;
    case "success":
      return isConsole ? console.log(`${newLine ? "\n" : ""} ${green("\u2714")} ${msg}`) : `${newLine ? "\n" : ""} ${green("\u2714")} ${msg}`;
    case "error":
      return isConsole ? console.error(` ${newLine ? "\n" : ""} ${red("\u274C")} ${msg}`) : `${newLine ? "\n" : ""} ${red("\u274C")} ${msg}`;
    default:
      return isConsole ? console.log(`${newLine ? "\n" : ""} ${dim("\u2755")} ${msg}`) : `${newLine ? "\n" : ""} ${dim("\u2755")} ${msg}`;
  }
};

class CliError extends Error {
}
const indent = " ".repeat(4);
const handleCliError = (error) => {
  if (error instanceof Error && !(error instanceof CliError)) {
    if (error.stack) {
      log({
        type: "error",
        msg: error.stack.split("\n").slice(1).join("\n")
      });
    }
    log({
      type: "error",
      msg: `
${indent}${dim(`${PACKAGE_NAME} v${VERSION}`)}`
    });
    log({
      type: "error",
      msg: `
${indent}Please open a Bug report with the information above:`
    });
    log({
      type: "error",
      msg: `${indent}https://github.com/${AUTHOR.name}/${PACKAGE_NAME}/issues/new`
    });
  }
};

const execa = promisify(exec);
const commiter = async () => {
  intro(lightYellow("Commitizen CLI"));
  const values = await group(
    {
      type: () => type(),
      message: () => message()
    },
    {
      onCancel: () => {
        cancel(CANCELED_OP_MSG);
        process.exit(0);
      }
    }
  );
  try {
    const commit = `${values.type}: ${values.message}`;
    const cmd = `git commit -m "${commit}"`;
    const { stdout: stdoutStatus, stderr: stderrStatus } = await execa(
      "git status"
    );
    if (stderrStatus)
      throw new CliError(`An error occured: ${stderrStatus}`);
    if (stdoutStatus.includes("no changes added to commit")) {
      const addStagedFiles = await confirm({
        message: "No changes added to commit, would you like to add modified files ?",
        initialValue: true
      });
      if (addStagedFiles) {
        const { stderr: stderrAdd } = await execa("git add .");
        if (stderrAdd)
          throw new CliError(`An error occured: ${stderrAdd}`);
        const { stderr: stderrCmd2 } = await execa(cmd);
        if (stderrCmd2)
          throw new CliError(`An error occured: ${stderrCmd2}`);
        outro(dedent`
        You're all set 🎉

        use "git push" to publish your local commits
      `);
      }
      return;
    }
    const { stderr: stderrCmd } = await execa(cmd);
    if (stderrCmd)
      throw new CliError(`An error occured: ${stderrCmd}`);
    outro(dedent`
    You're all set 🎉

    use "git push" to publish your local commits
  `);
  } catch (err) {
    log({ type: "error", msg: err.message });
    handleCliError(err);
    process.exit(1);
  }
};

cli(
  {
    name: PACKAGE_NAME,
    version: VERSION,
    commands: []
  },
  () => {
    commiter().catch((err) => {
      log({ type: "error", msg: err.message });
      handleCliError(err);
      process.exit(1);
    });
  }
);
